#
# adoomTools.py
#
# Copyright (c) 2002 The University of Utah and the Flux Group.
# All rights reserved.
#
# This file is part of the Janos Bees Software.
#
# Permission to use, copy, modify, distribute, and sell this software and
# its documentation is hereby granted without fee, provided that the above
# copyright notice and this permission/disclaimer notice is retained in all
# copies or modified versions, and that both notices appear in supporting
# documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
# WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
# ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
# 
# Users are requested, but not required, to send to csl-dist@cs.utah.edu any
# improvements that they make and grant Univ. of Utah redistribution rights.
#

import os

from beesUtil import *

# Map of capabilities stored in the kernels to arguments passed to the new
# flow/domain/whatever
adoomNamed = {
    "user/external-channels" : "adoom/echannels",
    # ^^^ domain name           ^^^ kernel name
    "user/neighbors" : "adoom/neighbors",
    "user/basicTimerFrequency" : "adoom/protocol/tf",
    "node" : "adoom/node",
    "logger" : "adoom/protocol/logger.DBG",
    
    "dlversion" : "adoom/protocol/dlversion",
    "idversion" : "adoom/protocol/idversion",
    "unknownVersion" : "adoom/protocol/unknownVersion",
    }

# Load the caps
def loadAdoomCapabilities(nodes):
    map(lambda node: node.capabilityMap().create("adoom"), nodes)
    map(lambda node: node.capabilityMap("adoom").readFrom("adoom.cap"),
        nodes)

def startAdoom(nodes):
    # Create the app flow
    map(lambda node, id: (node.bdomainRegister("primary").bdomain().
                          create("adoom/protocol/ec",
                                 # ^^^ The flows communication channel
                                 "bees.core.ProtocolBDomain",
                                 # ^^^ The main class
                                 args=["apps.adoom.ADoomProtocol"]
                                 #      ^^^ The app to start up
                                 )),
        nodes, range(0, len(nodes)))
    # Pass caps and set the flow in motion
    map(lambda node: (node.eventChannel("adoom/protocol/ec").
                      #                  ^^^ Flow to talk to
                      passCapability(named = adoomNamed,
                                     # ^^^ The named caps to pass through
                                     extras = ["adoom/protocol/cs"
                                               # ^^^ Extra/unnamed caps
                                               ])),
        nodes)

def fdoomString(peer):
    return peer["address"] + ":" + `peer["port"]`

def fdoomChannel(node, name, peer, servers):
    server = servers[fdoomString(peer)]
    (node.externalChannel().
     create(name,
            "if/ipv4/udp",
            ("*:"
             + `server["port"]`
             + ":"
             + server["address"]
             + ":"
             + `peer["port"]`),
            "if/ipv4/udp",
            ("*:"
             + `peer["port"]`
             + ":"
             + peer["address"]
             + ":"
             + `server["port"]`)))

def adoomUDPNeighbor(node, name, adoom, peer):
    createUDPNeighbor(node,
                      name,
                      localAddress = adoom["address"],
                      localPort = adoom["port"],
                      neighborAddress = peer["address"],
                      neighborPort = peer["port"])

def startFDoom(peers,
               ticrate=1000 / 30,
               timeout=30,
               servers={},
               command="fdoom"):
    speers = []
    for peer in peers:
        speers.append(fdoomString(peer))
    for peer in peers:
        speer = fdoomString(peer)
        args = [ command,
                 "-millispertic", `ticrate`,
                 "-timeout", `timeout`,
                 "-myaddr", speer,
                 ]
        if len(servers) > 0:
            args.append("-adoomserver")
            args.append(fdoomString(servers[speer]))
        args.extend(speers)
        print "Starting " + command + " " + `args`
        if os.fork() == 0:
            os.execvp(command, args);
