#
# authTools.py
#
# Copyright (c) 2002 The University of Utah and the Flux Group.
# All rights reserved.
#
# This file is part of the Janos Bees Software.
#
# Permission to use, copy, modify, distribute, and sell this software and
# its documentation is hereby granted without fee, provided that the above
# copyright notice and this permission/disclaimer notice is retained in all
# copies or modified versions, and that both notices appear in supporting
# documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
# WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
# ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
# 
# Users are requested, but not required, to send to csl-dist@cs.utah.edu any
# improvements that they make and grant Univ. of Utah redistribution rights.
#

from beesShell import *

def loadAuthCapabilities(nodes,
                         directory="auth",
                         filename="auth.cap",
                         password=""):
    map(lambda node, directory:
        node.capabilityMap().create(directory),
        nodes,
        [directory] * len(nodes))
    map(lambda node, directory, filename, password:
        node.capabilityMap(directory).readFrom(filename, password),
        nodes,
        [directory] * len(nodes),
        [filename] * len(nodes),
        [password] * len(nodes))

def createAuthMaterial(nodes, nodeMap):
    map(lambda node, id:
        (node.nodeOSProtocolEnumeration().
         create("comm/protocol-enum",
                protocols = [ "if/ipv4/udp" ]),
         node.addressRange().create("comm/address-range",
                                    rangePermissions = {
        "in-udp-destination-port" : "2048-65535",
        "in-udp-source-port" : "0-65535",
        "out-udp-source-port" : "2048-65535",
        "out-udp-destination-port" : "0-65535",
        },
                                    ipPermissions = {
        "in-udp-source-address" : "127.0.0.1",
        "in-udp-destination-address" : "127.0.0.1",
        "out-udp-source-address" : "127.0.0.1",
        "out-udp-destination-address" : "127.0.0.1",
        }),
         createUDPMulticastNeighbor(node,
                                    "comm/"
                                    + id
                                    + "/nics/0/broadcastNeighbor",
                                    8000),
         node.file().create("comm/" + id + "/nics/0/local-material",
                            id + ".local.mat"),
         node.file().create("comm/" + id + "/nics/0/advertise-material",
                            id + ".remote.mat")),
        nodes,
        nodeMap)

def computeAuthCapabilities(directory="auth",
                            discoveryDirectory="discovery",
                            livenessDirectory="liveness"):
    retval = {
        "logger" : directory + "/application/logger.DBG",
        "node" : "node",
        "unknownVersion" : directory + "/protocol/unknownVersion",
        "idversion" : directory + "/protocol/idversion",
        "dlversion" : directory + "/protocol/dlversion",
        "user/clock" : "clock",
        
        "user/meters/cpu-meter" : "/cpu-meter",
        "user/meters/mem-meter" : "/mem-meter",
        
        "user/protocol/unknownVersion" :
        directory + "/protocol/unknownVersion",
        "user/protocol/idversion" : directory + "/protocol/idversion",
        "user/protocol/dlversion" : directory + "/protocol/dlversion",
        "user/protocol/classes" : directory + "/protocol/cs",
        "user/protocol/log" : directory + "/protocol/logger.DBG",
        
        "user/liveness/logger" : livenessDirectory + "/protocol/logger.DBG",
        "user/liveness/classes" : livenessDirectory + "/protocol/cs",
        "user/liveVersion" : livenessDirectory + "/protocol/version",
        
        "user/discovery/logger" : discoveryDirectory + "/protocol/logger.DBG",
        "user/discovery/classes" : discoveryDirectory + "/protocol/cs",
        "user/discoveryVersion" : discoveryDirectory + "/protocol/version",

        "user/classes" : "classpath/bees.jar",
        
        "user/nic" : "comm/nics",

        "user/users" : "users",

        "user/global/nodeInfo" : "/nodeInfo",
        "user/global/nodeKeys" : "/nodeKeys",
    }
    return retval

def nodeAuthMaterialMap(name, authNamed):
    retval = {}
    retval.update(authNamed)
    retval.update({
        "user/nic" : "comm/" + name + "/nics"
        })
    return retval

def startSingleAuth(node,
                    authenticator="apps.authenticator.Everything",
                    directory="auth"):
    # Create the app flow
    (node.bdomainRegister("primary").bdomain().
     create(directory + "/ec",
            "bees.core.ApplicationBDomain",
            args=[authenticator],
            capabilities=["cap-meter", "cpu-meter", "mem-meter"],
            countPermissions={
            "cpu.shares" : 100,
            "cpu.threads" : 2,
            "mem.pages" : 1 * 1024 * 1024,
        }))
    name = node.node("node").getName()
    # Pass caps and set the flow in motion
    (node.eventChannel(directory + "/ec").
     passCapability(named = computeAuthCapabilities(),
                    extras = [directory + "/application/cs"]))

authNamedCapabilityMap = {
    "logger" : "auth/application/logger.DBG",
    "node" : "auth/node",
    "unknownVersion" : "auth/protocol/unknownVersion",
    "idversion" : "auth/protocol/idversion",
    "dlversion" : "auth/protocol/dlversion",

    "user/protocol/unknownVersion" :
    "auth/protocol/unknownVersion",
    "user/protocol/idversion" : "auth/protocol/idversion",
    "user/protocol/dlversion" : "auth/protocol/dlversion",
    "user/protocol/classes" : "auth/protocol/cs",
    "user/protocol/log" : "auth/protocol/logger.DBG",
    
    "user/liveness/logger" : "liveness/protocol/logger.DBG",
    "user/liveness/classes" : "liveness/protocol/cs",
    "user/liveVersion" : "liveness/protocol/version",
    
    "user/discovery/logger" : "discovery/protocol/logger.DBG",
    "user/discovery/classes" : "discovery/protocol/cs",
    "user/discoveryVersion" : "discovery/protocol/version",
    
    "user/comm/protocols" : "comm/protocol-enum",
    "user/comm/addresses" : "comm",
    }

def startAuth(nodes, nodeMap):
    # Create the app flow
    map(lambda node: (node.bdomainRegister("primary").bdomain().
                      create("auth/application/ec",
                             "bees.core.ApplicationBDomain",
                             args=["apps.authenticator.Everything", "DSA"],
                             capabilities=[])),
        nodes)
    # Pass caps and set the flow in motion
    map(lambda node, id:
        (node.eventChannel("auth/application/ec").
         passCapability(named = nodeAuthMaterialMap(id,
                                                    authNamedCapabilityMap),
                        extras = ["auth/application/cs"])),
        nodes, nodeMap)
