#
# materialTools.py
#
# Copyright (c) 2002 The University of Utah and the Flux Group.
# All rights reserved.
#
# This file is part of the Janos Bees Software.
#
# Permission to use, copy, modify, distribute, and sell this software and
# its documentation is hereby granted without fee, provided that the above
# copyright notice and this permission/disclaimer notice is retained in all
# copies or modified versions, and that both notices appear in supporting
# documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
# WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
# ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
# 
# Users are requested, but not required, to send to csl-dist@cs.utah.edu any
# improvements that they make and grant Univ. of Utah redistribution rights.
#

from bees.tools.beesUtil import *

def createDiscoveryNeighborMaterial(an,
                                    filePrefix,
                                    ipAddr = "127.0.0.1",
                                    udpPort = 4000,
                                    deleteFiles = 1):
    matNamed = {
        "logger" : "mat/logger.DBG",
        "node" : "node",
        "user/file" : "mat/remote.file",
        }
    
    localMatNamed = {
        "logger" : "mat/logger.DBG",
        "node" : "node",
        "user/file" : "mat/local.file",
        }

    try:
        an.capabilityMap().create("mat")

        an.file().create("mat/remote.file", filePrefix + ".remote.mat")
        an.file().create("mat/local.file", filePrefix + ".local.mat")
        
        an.systemClass().create("mat/cs", "bees.core.ApplicationBDomain")
        
        an.logger().create("mat/logger.DBG", "Material Builder")

        an.versionString().create("mat/unknownVersion", "Unknown 2002")
        an.versionString().create("mat/dlversion", "DLProtocol 2002")
        an.versionString().create("mat/idversion", "IDProtocol 2002")
        
        createPrimaryBDomainRegister([an])
        
        (an.bdomainRegister("primary").bdomain().
         create("mat/ec" + `udpPort`,
                "bees.core.ApplicationBDomain",
                args=["apps.authenticator.MaterialBuilder",
                      "-start-permission-set", "",
                      "-ip-permission", "in-source-address=" + ipAddr,
                      "-int-permission", "in-source-udp-port=" + `udpPort`,
                      "-ip-permission", "out-destination-address=" + ipAddr,
                      "-int-permission", "out-destination-udp-port=" + `udpPort`,
                      "-end-permission-set", "",],
                capabilities=["cpu-meter", "mem-meter"],
                countPermissions={
            "cpu.shares" : 100,
            "cpu.threads" : 5,
            "mem.pages" : 1 * 1024 * 1024,
            }))
        an.eventChannel("mat/ec" + `udpPort`).passCapability(named = matNamed,
                                                             extras = ["mat/cs"])
        try:
            an.bdomainRegister("primary").bdomain("mat/ec" + `udpPort`).waitFor()
        except MessageError:
            pass
        
        (an.bdomainRegister("primary").bdomain().
         create("mat/ec" + `udpPort`,
                "bees.core.ApplicationBDomain",
                args=["apps.authenticator.MaterialBuilder",
                      "-start-permission-set", "",
                      "-ip-permission", "in-destination-address=" + ipAddr,
                      "-int-permission", "in-destination-udp-port=" + `udpPort`,
                      "-ip-permission", "out-source-address=" + ipAddr,
                      "-int-permission", "out-source-udp-port=" + `udpPort`,
                      "-end-permission-set", "",],
                capabilities=["cpu-meter", "mem-meter"],
                countPermissions={
            "cpu.shares" : 100,
            "cpu.threads" : 5,
            "mem.pages" : 1 * 1024 * 1024,
            }))
        an.eventChannel("mat/ec"
                        + `udpPort`).passCapability(named = localMatNamed,
                                                    extras = ["mat/cs"])
        try:
            an.bdomainRegister("primary").bdomain("mat/ec" + `udpPort`).waitFor()
        except MessageError:
            pass
    finally:
        try:
            an.capabilityMap("mat").delete()
        except:
            pass

