#! /bin/sh
#
# TestScript.sh
#
# Copyright (c) 2002 The University of Utah and the Flux Group.
# All rights reserved.
#
# This file is part of the Janos Bees Software.
#
# Permission to use, copy, modify, distribute, and sell this software and
# its documentation is hereby granted without fee, provided that the above
# copyright notice and this permission/disclaimer notice is retained in all
# copies or modified versions, and that both notices appear in supporting
# documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
# WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
# ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
# 
# Users are requested, but not required, to send to csl-dist@cs.utah.edu any
# improvements that they make and grant Univ. of Utah redistribution rights.
#

# Get the test name
file_root=$1

# Start the JVM
$BUILDVM edu.utah.janos.nodeos.Main --quiet bees.core.bdomain.BDomain -tcpport 23000 > ${file_root}.vm 2>&1 &
vm_pid=$!

# Start the test script
env PYTHONPATH=$TOPDIR/pylib JSIDST=tcp:localhost/23000 $PYTHON ${TOPDIR}/testsuite/${file_root}.test > ${file_root}.script 2>&1
py_rc=$?

if test $py_rc -gt 0; then
    kill $vm_pid
else
    wait
fi

# Cat the outputs
cat $TOPDIR/testsuite/vmout.header ${file_root}.vm $TOPDIR/testsuite/scriptout.header ${file_root}.script > ${file_root}.tmp

EXTRA_FILES=*.extra

if test "$EXTRA_FILES" != "*.extra"; then
    for EXTRA in "$EXTRA_FILES"; do
	sed "s/@filename@/$EXTRA/g" $TOPDIR/testsuite/fileout.header | cat - $EXTRA >> ${file_root}.tmp
	rm -f $EXTRA
    done
fi

if test $py_rc -gt 0; then
    echo "FAIL: ${file_root}"
    cp -f ${file_root}.tmp ${file_root}.fail
elif test ! -f "${TOPDIR}/testsuite/${file_root}.out"; then
    # No archetype
    echo "PASS: ${file_root}"
    cp -f ${file_root}.tmp ${file_root}.out
elif cmp -s "${file_root}.tmp" "${TOPDIR}/testsuite/${file_root}.out"; then
    echo "PASS: ${file_root}"
else
    echo "FAIL: ${file_root}"
    cp -f ${file_root}.tmp ${file_root}.fail
fi

rm -f ${file_root}.tmp
