#
# configuration-help.py
#
# Copyright (c) 2002 The University of Utah and the Flux Group.
# All rights reserved.
#
# This file is part of the Janos Bees Software.
#
# Permission to use, copy, modify, distribute, and sell this software and
# its documentation is hereby granted without fee, provided that the above
# copyright notice and this permission/disclaimer notice is retained in all
# copies or modified versions, and that both notices appear in supporting
# documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
# WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
# ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
# 
# Users are requested, but not required, to send to csl-dist@cs.utah.edu any
# improvements that they make and grant Univ. of Utah redistribution rights.
#

import os
import sys
import time
import beesPageTemplate

config = {
    "title" : "Configuration Help",
    "time" : time.ctime(os.stat(sys.argv[0])[8]),
    "help-file" : "root-help.html",
    }

body = """
<h2>Configuring a Node</h2>
<p>

An Bees node needs to be configured with network interfaces, keys, and an
authenticator before it can be made available to the rest of the network.  The
configuration page performs this task by first querying the JanosVM running on
the node to figure out the machine's name and active network interfaces.  This
information is then used to construct a form with a reasonable set of defaults
that you can then modify.  The form breaks down into four major parts:

<h4>Node Identity</h4>

<ul>
<li>Name - The name that is returned whenever a protocol or application calls
bees.core.Node.getName()
<li>Version - The version string that is returned whenever a protocol or
application calls bees.core.Node.getVersion()
</ul>

<h4>Authenticator</h4>

The authenticator application to be executed on the node.  This application
will decide whether a flow will be allowed onto the node and what capabilities
it should be given.  Currently, the possibilities are hard coded to the two
built-in authenticators, Everything and Source Based.

<h4>Communication</h4>

This section is used to configure the node's network interfaces, if you do not
want to configure an interface, simply delete the field's value.  Each active
interface is listed with the suggested values in two sections, broadcast and
peer to peer.  The broadcast section defines what multicast addresses to use
when sending and listening for node advertisements.  The suggested IP address
is an internal multicast address that will not be forwarded outside the LAN.
Because multicast ports can be shared, the same port number and IP can be
shared amongst the interfaces.  Next, the peer to peer section describes which
UDP ports are to be used when communicating with peers that are discovered on
the corresponding network interfaces.

<h4>Node Keys</h4>

Each node needs to have a unique set of cryptographic keys to sign and verify
messages from their peers.  This section allows you to specify the capability
file names and encryption passwords of the DSA and RSA key pairs for this node.
If the capability files do not exist or the fields are left blank, the keys
will automatically be generated and saved to the given file name.  In addition
to creating unique keys for this node, the configuration process will scan the
node's current directory for any files that end in ".dsa.cap" or ".rsa.cap" and
load them into the system under the "/nodeInfo" directory.  Unfortunately, Bees
does not have a key distribution protocol, so we use this process as a stopgap
measure for sharing keys between nodes.  Therefore, it is suggested that these
files be kept in an NFS'd directory or manually copied to each peer node.

<h2>Viewing the Current Configuration</h2>

Once a node has been configured, the configuration page will change to reflect
the current configuration on the node.  Currently, the page contains the
following sections:

<ul>
<li>Broadcast - The broadcast neighbors.  <i>Note</i>: These are listed using
NodeOS style protocol and address specs.
<li>Public Keys - The finger prints of the node's public keys.
<li>Port Map - The contents of the beesconfig.py file.
</ul>

"""

print beesPageTemplate.header % config

print body

print beesPageTemplate.footer % config
